/* 
 * Project name:
     HID_Read_Write (USB HID Read & Write Test)
 * Copyright:
     (c) Mikroelektronika, 2014.
 * Revision History:
     20140924:
       - initial release (FJ);
 * Description:
     This example establishes connection with the HID terminal that is active
     on the PC. Upon connection establishment, the HID Device Name will appear
     in the respective window. The character that user sends to ARM from the HID
     terminal will be re-sent back to user.
 * Test configuration:
     MCU:             STM32F407VG
                      http://www.st.com/st-web-ui/static/active/en/resource/technical/document/datasheet/DM00037051.pdf
     Dev.Board:       clicker 2 for STM32 - ac:clicker_2_STM32
                      http://www.mikroe.com/stm32/clicker-2/
     Oscillator:      HSI-PLL, 120.000MHz
     Ext. Modules:    None.
     SW:              mikroC PRO for ARM
                      http://www.mikroe.com/mikroc/arm/
 */
#include <built_in.h>
#include "ADS1220.h"

char readbuff[64];
char writebuff[64];
char SPI_MASTER_DUMMY;
int dFlag = 0;
sfr sbit m_cs_pin at GPIOE_ODR.B2;        // SPI chip select
sfr sbit testpin at GPIOE_ODR.B3;     // TEST PIN
//***********************************************************************************/
//InitMCU(): Initialises all of the required general purpose IO and SPI.
//Parameters: None
//***********************************************************************************/
InitMCU()
{
   Delay_ms(100);        // Delay after power-up for ADS1220 voltages to stabilise.
   //Setup Digital pins for SPI
   //GPIO_Digital_Output(&GPIOE_BASE, _GPIO_PINMASK_13);   //Reset pin
   //GPIO_Digital_Output(&GPIOE_BASE, _GPIO_PINMASK_8);   //SPI CS
   GPIO_Config(&GPIOE_BASE, _GPIO_PINMASK_2,
            _GPIO_CFG_MODE_OUTPUT | _GPIO_CFG_PULL_UP); //SPI CS
   GPIO_Digital_Input(&GPIOE_BASE, _GPIO_PINMASK_1);   //Interrupt pin (Don't Pull-up)

   GPIO_Digital_Output(&GPIOE_BASE, _GPIO_PINMASK_3);   // TEST PIN
   //GPIO_Digital_Output(&GPIOE_BASE, _GPIO_PINMASK_12);   //LED

   m_cs_pin = 1;                    // Deselect ADS1220
   testpin = 1;
   
   // Setup SPI. Use SPI3 module at PORTB[10:12]
   //SPI3_Init_Advanced(_SPI_FPCLK_DIV16, _SPI_MASTER  | _SPI_8_BIT |
   //                   _SPI_CLK_IDLE_low | _SPI_FIRST_CLK_EDGE_TRANSITION |
   //                   _SPI_MSB_FIRST | _SPI_SS_DISABLE | _SPI_SSM_ENABLE | _SPI_SSI_1,
   //                   &_GPIO_MODULE_SPI3_PC10_11_12);
   // Setup SPI. Use SPI1 module at PORTA[5:7]
   SPI1_Init_Advanced(_SPI_FPCLK_DIV16, _SPI_MASTER  | _SPI_8_BIT |
                      _SPI_CLK_IDLE_low | _SPI_FIRST_CLK_EDGE_TRANSITION |
                      _SPI_MSB_FIRST | _SPI_SS_DISABLE | _SPI_SSM_ENABLE | _SPI_SSI_1,
                      &_GPIO_MODULE_SPI1_PA567);

   //Setup External Interrupt
   SYSCFGEN_bit = 1;                    // Enable clock for alternate pin functions
   SYSCFG_EXTICR1 = 0x00000040;         // Map external interrupt on PE1
   EXTI_RTSR = 0x00000000;              // Set interrupt on Rising edge (none)
   EXTI_FTSR = 0x00000002;              // Set Interrupt on Falling edge (PE1)
   EXTI_IMR |= 0x00000002;              // Set mask
   NVIC_IntEnable(IVT_INT_EXTI1);   // Enable External interrupt
}
/* ADS1220 Initial Configuration */
void ADS1220Config(void)
{
   unsigned char Temp;
   // Register 0
   ADS1220ReadRegister(ADS1220_0_REGISTER, 0x01, &Temp);
   // clear prev value
   Temp &= 0x00;
   Temp |= (ADS1220_MUX_1_2 | ADS1220_GAIN_128);
   // write the register value containing the new value back to the ADS
   ADS1220WriteRegister(ADS1220_0_REGISTER, 0x01, &Temp);
   
   // Register 1
   ADS1220ReadRegister(ADS1220_1_REGISTER, 0x01, &Temp);
   // clear prev DataRate code;
   Temp &= 0x00;
   Temp |= (ADS1220_DR_330 | ADS1220_CC);                // Set default start mode to 330sps and continuous conversions
   // write the register value containing the new value back to the ADS
   ADS1220WriteRegister(ADS1220_1_REGISTER, 0x01, &Temp);

   // Register 2
   ADS1220ReadRegister(ADS1220_2_REGISTER, 0x01, &Temp);
   // clear prev DataRate code;
   Temp &= 0x00;
   Temp |= (ADS1220_VREF_EX_DED | ADS1220_IDAC_2000);      // External reference selected using dedicated REFP0 and REFN0 inputs
   // write the register value containing the new value back to the ADS
   ADS1220WriteRegister(ADS1220_2_REGISTER, 0x01, &Temp);
   Delay_us(200);      // IDACs require up to 200 s to start up

   // Register 3
   ADS1220ReadRegister(ADS1220_3_REGISTER, 0x01, &Temp);
   // clear prev DataRate code;
   Temp &= 0x00;
   Temp |= (ADS1220_IDAC1_AIN0);      // IDAC1 connected to AIN0/REFP1
   // write the register value containing the new value back to the ADS
   ADS1220WriteRegister(ADS1220_3_REGISTER, 0x01, &Temp);
}
void ADS1220SendByte(unsigned char Byte)
{        
   SPI1_Write(Byte);
}
unsigned char ADS1220ReceiveByte(void)
{
   unsigned char Result = 0;

   Result = SPI1_Read(SPI_MASTER_DUMMY);
   return Result;
}
/*
******************************************************************************
 higher level functions
*/
long ADS1220ReadData(void)
{
   long Data;
   // de-assert CS to start transfer
   m_cs_pin = 0;
   Delay_us(1);
   // send the command byte
   ADS1220SendByte(ADS1220_CMD_RDATA);
   // get the conversion result

   Data = ADS1220ReceiveByte();
   Data = (Data << 8) | ADS1220ReceiveByte();
   Data = (Data << 8) | ADS1220ReceiveByte();
   
   // sign extend data
   if (Data & 0x800000)
      Data |= 0xff000000;
   Delay_us(1);
   // assert CS
   m_cs_pin = 1;
   return Data;
}
void ADS1220ReadRegister(int StartAddress, int NumRegs, unsigned * pData)
{
   int i;
   // de-assert CS to start transfer
   //m_cs_pin = 0;
  
   // send the command byte
   ADS1220SendByte(ADS1220_CMD_RREG | (((StartAddress<<2) & 0x0c) |((NumRegs-1)&0x03)));

   // get the register content
   for (i=0; i< NumRegs; i++)
   {
      *pData++ = ADS1220ReceiveByte();
   }
   // assert CS
   //m_cs_pin = 1;
   return;
}
void ADS1220WriteRegister(int StartAddress, int NumRegs, unsigned * pData)
{
   int i;
   // de-assert CS to start transfer
   //m_cs_pin = 0;
   // send the command byte */
   ADS1220SendByte(ADS1220_CMD_WREG | (((StartAddress<<2) & 0x0c) |((NumRegs-1)&0x03)));
   // send the data bytes
   for (i=0; i< NumRegs; i++)
   {
      ADS1220SendByte(*pData++);
   }
   // assert CS
   //m_cs_pin = 1;
   return;
}
void ADS1220SendResetCommand(void)
{
   // de-assert CS to start transfer
   //m_cs_pin = 0;
   Delay_us(100);
   // send the command byte
   ADS1220SendByte(ADS1220_CMD_RESET);
   Delay_us(100);
   // assert CS
  // m_cs_pin = 1;
   return;
}
void ADS1220SendStartCommand(void)
{
   // de-assert CS to start transfer
   //m_cs_pin = 0;
   // send the command byte
   ADS1220SendByte(ADS1220_CMD_SYNC);
   Delay_us(100);
   // assert CS
   //m_cs_pin = 1;
   return;
}
//***********************************************************************************/
//StoreADC(): Converts long to byte array
//Parameters: None
//***********************************************************************************/

void StoreADS(signed long val, unsigned char pos){

  int i, j;
  union {
    signed long ADSVal;
    char bytes[4];
  } u;
  // Overwrite bytes of union with signed long variable
  u.ADSVal = val;

  // Store long int bytes into designated positions in writebuff
  for(i = 0; i<4; i++)
  {
     writebuff[pos] = u.bytes[i];
     pos++;
  }

}

//***********************************************************************************/
//ExtInt():
//Parameters: None
//***********************************************************************************/
void ExtInt() iv IVT_INT_EXTI1 ics ICS_AUTO
{
   if(EXTI_PR.B1)
   {
      EXTI_PR.B1 = 1;                    // clear flag
      testpin = 0;               // Togle Pin
      dFlag = 1;
   }
}

void USB0Interrupt() iv IVT_INT_OTG_FS
{
  USB_Interrupt_Proc();
}

void main(void){
   signed long tData;

   InitMCU();
   // de-assert CS to start transfer
   m_cs_pin = 0;                        // Set CS to the device low;
   ADS1220SendResetCommand();           // Send the RESET command
   ADS1220Config();                        // Set base configuration for ADS1x20
   ADS1220SendStartCommand();           // Send the START/SYNC command
   // assert CS
   m_cs_pin = 1;                        // Clear CS to high (resets the serial interface);
   
   memset(writebuff, 0, sizeof(writebuff));
   memset(readbuff, 0, sizeof(readbuff));
   HID_Enable(&readbuff,&writebuff);

   while(1)
   {
      //while(!HID_Read());
      HID_Read();                                   // Read from USB
      
      // Add specifc command for reading and writing ADS1220 here
      // dFlag is set in the interrupt service routine when DRDY triggers end of conversion
      //if (dFlag)                        // check if new data is available
      //{
         tData = ADS1220ReadData();        // get the data from the ADS1220
         StoreADS(tData,0);
      //   dFlag = 0;
      //   testpin = 1;
      //}
      // other routines could be added here, such as change the mux setting
      
      while(!HID_Write(&writebuff,64));
  }
}